#!/usr/bin/python
# -*- coding: utf-8 -*-

# Avalon RSS Notify
# Licensed under GPL

import settingsparser
import bimp
import feedparser

import sys
import os
import gtk
import pygtk
import gtk.glade
import gobject
import threading
import time
import pynotify

feedparser.USER_AGENT = 'AvalonRSS/0.2 +http://avalonrss.tk'
s = settingsparser.Config()

locked = False
place = sys.path[0]

orange_box = place + '/data/icons/orange-box.png'
grey_box = place + '/data/icons/grey-box.png'
orange_open = place + '/data/icons/orange-16.png'
grey_open = place + '/data/icons/grey-16.png'
orange_open_big = place + '/data/icons/orange-32.png'


def show_notify(header, text, icon):
  if __name__ == '__main__':
    if not pynotify.init("Basics"):
      sys.exit(1)

    n = pynotify.Notification(header, text, icon)
    n.set_urgency(pynotify.URGENCY_NORMAL)
    n.set_timeout(3)

    if not n.show():
      print "Failed to send notification"
      sys.exit(1)



def show_settings(widget):
	if not locked:
		sgui = SettingsGui()


def mark_all(widget):
	"""Adds all feed entries to the history file
	   and rebuilds the menu"""

	try:
		h = file(place + '/data/history', 'a')
	except:
		print 'Could not open history file'
		sys.exit(0)

	for feed in s.feeds():
		if feed.split(',')[2] == '1':
			f = rawdata[feed.split(',')[0]]

			if s.maxfeeds() > len(f.entries):
				shown_entries = len(f.entries)
			else:
				shown_entries = s.maxfeeds()

			for num in range(0,shown_entries):
				if history(f.entries[num].link):
					h.write(f.entries[num].link + '\n')
	h.close()
	BuildFeedmenu(False)

def history(link):
	"""Checks if link exists in the history file"""
	try:
		f = file(place + '/data/history', 'r')
	except:
		print 'Could not open history file'
		sys.exit(0)

	history = f.read().splitlines()
	f.close()

	if history.count(link) > 0:
		return False
	else:
		return True


def update_feedmenu(widget):
	global rawdata

	rawdata = fetch_feeds()
	BuildFeedmenu(s.notify())

	return True

def popup_menu_cb(widget, button, time, data = None):
	"""Connects the menu to the status icon"""
	if button == 3:
		if data:
			data.show_all()
			data.popup(None, None, None, 3, time)
	pass


def fetch_feeds():
	"""Loads and parse the feeds"""

	class ThreadClass(threading.Thread):
		def __init__ (self, link):
			threading.Thread.__init__(self)
			self.link = link

		def run(self):
			self.feed = feedparser.parse(self.link)


	start_time = time.time()
	threadlist = []
	for feedlink in s.feeds():

		if feedlink.split(',')[2] == '1':
			t = ThreadClass(feedlink.split(',')[0])
			threadlist.append(t)
			t.start()

	data ={}
	for item in threadlist:
		item.join()
		data[item.link] = item.feed

		if not data[item.link].feed:
			print 'Could not fetch feed from ' + item.link

			tmp_feed = s.feeds()

			count = 0
			for feed in tmp_feed:
				if feed.split(',')[0] == item.link:
					tmp_feed[count] = feed.replace('1', '0')
					s.Write('feeds', tmp_feed)

				count = count + 1

	seconds = str(time.time() - start_time)
	clock = time.strftime('%H:%M:%S')


	print 'Feeds fetched in ' + seconds + ' seconds at ' + clock
	return data



def new_callback():
	global cake

	gobject.source_remove(cake)
	cake = gobject.timeout_add_seconds(s.updateinterval(), update_feedmenu, None)

def decode_entities(text):
	"""Returns a string with decoded entities"""

	text = text.replace('&#34;', '"')
	text = text.replace('&#39;', '')
	text = text.replace('&#38;', '&')
	text = text.replace('&#60;', '<')
	text = text.replace('&#62;', '>')
	text = text.replace('&rdquo;', '"')
	text = text.replace('&ndash;', '-')
	text = text.replace('&amp;', '&')
	text = text.replace('&euro;', '€')
	text = text.replace('&#8230;', '...')
	text = text.replace('&#8220;', '“')
	text = text.replace('&#8221;', '”')
	return text

class SettingsGui:
	"""Creates a settings dialog"""

	def __init__(self):
		global locked
		locked = True

		#Loads gladefile
		self.gladefile = place + '/settings.glade'
		self.wTree = gtk.glade.XML(self.gladefile)

	    #Loads widgets
		self.win = self.wTree.get_widget('windowmain')
		self.notify = self.wTree.get_widget('ckbtn_notify')
		self.browser = self.wTree.get_widget('entry_browser')
		self.show = self.wTree.get_widget('spbtn_show')
		self.update = self.wTree.get_widget('spbtn_update')
		self.tree = self.wTree.get_widget('treeview1')
		self.dia = self.wTree.get_widget('dia')
		self.name = self.wTree.get_widget('ent_name')
		self.url = self.wTree.get_widget('ent_url')

		#Loads values from config
		self.notify.set_active(s.notify())
		self.browser.set_text(s.browser()[0])
		self.show.set_value(s.maxfeeds())
		self.update.set_value(s.updateinterval() / 60)

		#Creates a model for the treeview
		store = self.create_model(s.feeds())
		self.treeView = self.tree
		self.treeView.set_model(store)
		self.treeView.set_rules_hint(True)
		self.create_columns(self.treeView)


		#Creates a dictionary with all signals
		dic = { 'on_btn_dia_add_clicked':        self.dia_add_clicked,
				'on_btn_dia_exit_clicked':       self.dia_exit_clicked,
				'on_btn_add_clicked':            self.add_clicked,
				'on_btn_remove_clicked':         self.remove_clicked,
				'on_btn_import_clicked':	     self.import_clicked,
				'on_btn_ok_clicked':             self.ok_clicked,
				'on_btn_apply_clicked':          self.apply_clicked,
				'on_btn_quit_clicked':           self.close_settings,
				'on_windowmain_destroy':         self.close_settings}

		self.wTree.signal_autoconnect(dic)

	def dia_add_clicked(self, widget):
		url_list = []
		for nam in self.treeView.get_model():
			show = '0'
			if nam[0]:
				show = '1'
			url_list.append(nam[2] + ',' + nam[1] + ',' + show)

		url_list.append(self.url.get_text() + ',' + self.name.get_text() + ',0')
		store = self.create_model(url_list)
		self.treeView.set_model(store)

		self.name.set_text('')
		self.url.set_text('')
		self.dia.hide()


	def dia_exit_clicked(self, widget):
		self.dia.hide()


	def add_clicked(self, widget):
		self.dia.show()


	def remove_clicked(self, widget):
		try:
			liststore = self.treeView.get_selection().get_selected()[0]
			iter = self.treeView.get_selection().get_selected()[1]

			liststore.remove(iter)
		except:
			pass


	def close_settings(self, widget):
		global locked
		self.win.destroy()
		locked = False


	def import_clicked(self, widget):
		dialog = gtk.FileChooserDialog('Choose file to import',
		None,
		gtk.FILE_CHOOSER_ACTION_OPEN,
		(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
		gtk.STOCK_OPEN, gtk.RESPONSE_OK))
		dialog.set_default_response(gtk.RESPONSE_OK)

		filter = gtk.FileFilter()
		filter.set_name('Bookmarks')
		filter.add_mime_type('text/html')
		filter.add_pattern('*.htm')
		filter.add_pattern('*.html')
		filter.add_pattern('*.xml')
		filter.add_pattern('*.json')
		dialog.add_filter(filter)

		filter = gtk.FileFilter()
		filter.set_name('All files')
		filter.add_pattern('*')
		dialog.add_filter(filter)

		response = dialog.run()
		if response == gtk.RESPONSE_OK:
			ib = bimp.import_bookmarks(dialog.get_filename())

			feed_list = []
			url_list = []

			for nam in self.treeView.get_model():
				url_list.append(nam[2])
				visible = '0'
				if nam[0]:
					visible = '1'
				feed_list.append(nam[2] + ',' + nam[1] + ',' + visible)

			for formated_feed in ib.make_list():
				feed_link = formated_feed.split(',')[0]

				if not feed_link in url_list and not formated_feed in feed_list:
					name = formated_feed.split(',')[1]
					formated_feed = formated_feed.replace(name, decode_entities(name))
					feed_list.append(formated_feed)

			store = self.create_model(feed_list)
			self.treeView.set_model(store)

		elif response == gtk.RESPONSE_CANCEL:
			print 'Closed, no file selected'

		dialog.destroy()

	def ok_clicked(self, widget):
		self.apply_clicked(None)
		self.close_settings(None)


	def apply_clicked(self, widget):

		if int(self.update.get_value()) * 60 != s.updateinterval():
			new_callback()

		s.Write('updateinterval', int(self.update.get_value()))
		s.Write('maxfeeds', int(self.show.get_value()))
		s.Write('browser', self.browser.get_text())
		s.Write('notify', self.notify.get_active())

		tmp_urls = []
		for nam in self.treeView.get_model():
			visible = '0'
			if nam[0]:
				visible = '1'

			tmp_urls.append(nam[2] + ',' + nam[1] + ',' + visible)

		if s.feeds() == tmp_urls:
			BuildFeedmenu(None)

		else:
			s.Write('feeds', tmp_urls)
			update_feedmenu(None)


	def create_model(self, feedlist):
		store = gtk.ListStore(gobject.TYPE_BOOLEAN, str, str)
		for feed in feedlist:
			if feed.split(',')[2] == '1':
				store.append([True, feed.split(',')[1], feed.split(',')[0]])
			else:
				store.append([False, feed.split(',')[1], feed.split(',')[0]])
		return store


	def create_columns(self, treeView):
		rendererToogle = gtk.CellRendererToggle()
		rendererToogle.set_property('activatable', True)
		rendererToogle.connect('toggled', self.on_toggle_toggled)
		column = gtk.TreeViewColumn('Show', rendererToogle, active=0)
		treeView.append_column(column)

		rendererText = gtk.CellRendererText()
		rendererText.set_property('editable', True)
		rendererText.connect('edited', self.on_cell_clicked, 1)
		column = gtk.TreeViewColumn('Name', rendererText, text=1)
		column.set_sort_column_id(1)
		column.set_fixed_width(120)
		column.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
		column.set_resizable(True)
		treeView.append_column(column)

		rendererText = gtk.CellRendererText()
		rendererText.set_property('editable', True)
		rendererText.connect('edited', self.on_cell_clicked, 2)
		column = gtk.TreeViewColumn('Address', rendererText, text=2)
		column.set_sort_column_id(2)
		treeView.append_column(column)


	def on_cell_clicked(self, widget, row, text, col):
		model = self.treeView.get_model()
		iter = model.get_iter(row)
		model.set(iter, col, text)


	def on_toggle_toggled(self, widget, row):
		model = self.treeView.get_model()
		iter = model.get_iter(row)

		val = model.get_value(iter, 0)
		val = not val

		model.set(iter, 0, val)

class Menu():
	"""Creates a status icon with a empty menu"""

	def __init__(self):
		self.menu = gtk.Menu()
		self.statusicon = gtk.StatusIcon()
		self.statusicon.connect('popup-menu', popup_menu_cb, self.menu)
		self.sublist = []


	def set_statusicon(self, icon, tooltip):
		"""Adds a tooltip and a icon to the status icon"""
		self.statusicon.set_tooltip(tooltip)
		self.statusicon.set_from_file(icon)


	def add_menuitem(self, label, action):
		"""Adds a menuitem to the basic menu"""
		menuitem = gtk.MenuItem(label)
		menuitem.connect('activate', action)

		self.menu.append(menuitem)
		return menuitem


	def add_submenuitem(self, submenu, label, icon, action, link, tooltip):
		"""Adds a menuitem to the choosen submenu"""
		submenuitem = gtk.ImageMenuItem(label)
		submenuitem.connect('activate', action, link)

		img = gtk.Image()
		img.set_from_file(icon)
		submenuitem.set_image(img)
		submenuitem.set_tooltip_text(tooltip)

		submenu.append(submenuitem)
		return submenuitem


	def add_subseparator(self, submenu):
		"""Adds a separator to the choosen submenu"""
		separator = gtk.SeparatorMenuItem()
		submenu.append(separator)

	def add_separator(self):
		"""Adds a separator to the basic menu"""
		separator = gtk.SeparatorMenuItem()
		self.menu.append(separator)


	def add_submenu(self, label, icon):
		"""Adds a menuitem with a submenu to the basic menu"""
		submenuitem = gtk.ImageMenuItem(label)

		img = gtk.Image()
		img.set_from_file(icon)
		submenuitem.set_image(img)

		submenu = gtk.Menu()
		submenuitem.set_submenu(submenu)
		self.menu.append(submenuitem)

		self.sublist.append(submenuitem)
		return submenu

	def clear_submenus(self):
		"""Removes all submenus in the basic menu"""
		for sub in self.sublist:
			sub.destroy()

		self.sublist = []


class BuildFeedmenu():
	"""Creates one submenu in the basic menu for each feed in the config file"""

	def __init__(self, notify):
		menu.clear_submenus()
		self.new_feeds = 0

		notifydic = {}
		for feed in s.feeds():
			show = feed.split(',')[2] == '1'

			if show:
				f = rawdata[feed.split(',')[0]]
				notifydic[feed.split(',')[1]] = 0

				if s.maxfeeds() > len(f.entries):
					shown_entries = len(f.entries)
				else:
					shown_entries = s.maxfeeds()

				icon = grey_box
				for num in range(0,shown_entries):
					if history(f.entries[num].link):
						icon = orange_box
						notifydic[feed.split(',')[1]] = notifydic[feed.split(',')[1]] + 1

				feedsubmenu = menu.add_submenu(feed.split(',')[1], icon)
				for num in range(0,shown_entries):

					entrytitle = f.entries[num].title
					link = f.entries[num].link
					tooltip = self.tooltip(f.entries[num].summary)
					icon = grey_box

					if history(link):
						icon = orange_box
						self.new_feeds = self.new_feeds + 1

					menu.add_submenuitem(feedsubmenu, entrytitle,
										icon, self.entry_clicked, link, tooltip)

				menu.add_subseparator(feedsubmenu)

				feedname = feed.split(',')[1]
				link = f.feed.link
				menu.add_submenuitem(feedsubmenu, 'Open "' + feedname + '"',
									None, self.entry_clicked, link, None)

		if self.new_feeds > 0:
			menu.set_statusicon(orange_open, str(self.new_feeds) + ' New Items')

			#TODO: Improve notify layout and move formating to show_notify()
			if notify:
				notify_text = ''
				for name in notifydic:
					if notifydic[name] > 0:
						new_items = str(notifydic[name])
						feed_line = new_items + ' New items from ' + name + '\n'
						notify_text = notify_text + feed_line

				notify_text	= notify_text[:-1]
				show_notify('Avalon RSS Notify', notify_text, orange_open_big)
		else:
			menu.set_statusicon(grey_open, '0 New Items')


	def tooltip(widget, summary):
		"""Creates a tooltip with wordwrap and decoded entities"""

		words = summary.split(' ')
		tooltip = ''
		linenum = 0

		for count in range(0, 12):
			try:
				tooltip = tooltip + words[count]  + ' '

				if len(tooltip.split('\n')[linenum]) >= 35:
					tooltip = tooltip.strip() + '\n'
					linenum = linenum + 1
			except:
				pass

		tooltip = decode_entities(tooltip)
		tooltip = tooltip.strip()
		if not tooltip.endswith('.') and len(tooltip) > 0:
			tooltip = tooltip + '...'

		return tooltip

	def entry_clicked(self, submenuitem, link):
		"""Adds the choosen entry to the history file, changes icon to grey
		   updates the status icon and opens the choosen entry in a browser"""

		if history(link):
			try:
				f = file(place + '/data/history', 'a')
			except:
				print 'Could not open history file'
				sys.exit(0)

			f.write(link + '\n')
			f.close()

			BuildFeedmenu(False)
		s.browser()[1].open(link)

def about_dialog(widget):
	dia = gtk.AboutDialog()

	logo = gtk.gdk.pixbuf_new_from_file(place + '/data/icons/icon.svg')
	dia.set_logo(logo)

	dia.set_name('Avalon RSS')
	dia.set_version('0.2')
	dia.set_comments('A simple RSS reader for Ubuntu')
	dia.set_copyright('Copyright © 2010 Andreas Dahlberg')
	dia.set_website('http://avalonrss.tk')

	authors = ['Andreas Dahlberg']
	dia.set_authors(authors)
	artists = ['Andreas Dahlberg', 'Icejoy']
	dia.set_artists(artists)

	response = dia.run()
	if response == gtk.RESPONSE_CANCEL or response == gtk.RESPONSE_DELETE_EVENT:
		dia.destroy()



menu = Menu()
menu.add_menuitem('Properties', show_settings)
menu.add_menuitem('Mark all as read', mark_all)
menu.add_menuitem('Update', update_feedmenu)
menu.add_separator()

update_feedmenu(None)

cake = gobject.timeout_add_seconds(s.updateinterval(), update_feedmenu, None)
gtk.main()
